# Taxonomy related to writing characters in cursive in the style of the Shenoy
# lab.
from enum import IntEnum, auto


class Character(IntEnum):
    a = 0
    b = auto()
    c = auto()
    d = auto()
    e = auto()
    f = auto()
    g = auto()
    h = auto()
    i = auto()
    j = auto()
    k = auto()
    l = auto()
    m = auto()
    n = auto()
    o = auto()
    p = auto()
    q = auto()
    r = auto()
    s = auto()
    t = auto()
    u = auto()
    v = auto()
    w = auto()
    x = auto()
    y = auto()
    z = auto()
    apostrophe = auto()
    comma = auto()
    doNothing = auto()
    greaterThan = auto()
    questionMark = auto()
    tilde = auto()


class Line(IntEnum):
    straightLine_short_1 = 0
    straightLine_short_2 = auto()
    straightLine_short_3 = auto()
    straightLine_short_4 = auto()
    straightLine_short_5 = auto()
    straightLine_short_6 = auto()
    straightLine_short_7 = auto()
    straightLine_short_8 = auto()
    straightLine_short_9 = auto()
    straightLine_short_10 = auto()
    straightLine_short_11 = auto()
    straightLine_short_12 = auto()
    straightLine_short_13 = auto()
    straightLine_short_14 = auto()
    straightLine_short_15 = auto()
    straightLine_short_16 = auto()
    straightLine_medium_1 = auto()
    straightLine_medium_2 = auto()
    straightLine_medium_3 = auto()
    straightLine_medium_4 = auto()
    straightLine_medium_5 = auto()
    straightLine_medium_6 = auto()
    straightLine_medium_7 = auto()
    straightLine_medium_8 = auto()
    straightLine_medium_9 = auto()
    straightLine_medium_10 = auto()
    straightLine_medium_11 = auto()
    straightLine_medium_12 = auto()
    straightLine_medium_13 = auto()
    straightLine_medium_14 = auto()
    straightLine_medium_15 = auto()
    straightLine_medium_16 = auto()
    straightLine_long_1 = auto()
    straightLine_long_2 = auto()
    straightLine_long_3 = auto()
    straightLine_long_4 = auto()
    straightLine_long_5 = auto()
    straightLine_long_6 = auto()
    straightLine_long_7 = auto()
    straightLine_long_8 = auto()
    straightLine_long_9 = auto()
    straightLine_long_10 = auto()
    straightLine_long_11 = auto()
    straightLine_long_12 = auto()
    straightLine_long_13 = auto()
    straightLine_long_14 = auto()
    straightLine_long_15 = auto()
    straightLine_long_16 = auto()
